#include <Wire.h>
#include "rgb_lcd.h"
#include "AirQuality.h"
#include "MHZ19.h"
#include "DHT.h"
#include "Arduino.h"

#define DHTTYPE DHT22 
#define DHTPIN 7     // Digital pin connected to the DHT sensor

// на скорости 9600 (8 bit, stop — 1, parity — none) нужно отправить следующие девять байт:
//• 0xFF — начало любой команды
//• 0x01 — первый сенсор (он всего один)
//• 0x86 — команда
//• 0x00, 0x00, 0x00, 0x00, 0x00 — данные
//• 0x79 — контрольная сумма.

//В ответ придет что-то такое:
//• 0xFF — начало любого ответа
//• 0x86 — команда
//• 0x01, 0xC1 — старшее и младшее значение (256 * 0x01 + 0xC1 = 449)
//• 0x3C, 0x04, 0x3C, 0xC1 — в документации сказано, что должно приходить что-то типа 0x47, 0x00, 0x00, 0x00, но на деле приходит непонятно что.
//• 0x7B — контрольная сумма.

// Контрольная сумма считается следующим образом: берутся 7 байт ответа (все кроме первого и последнего), складываются, инвертируются, увеличиваются на 1: 0x86 + 0x01… + 0xC1 = 0x85, 0x85 xor 0xFF = 0x7A, 0x7A + 1 = 0x7B.

/*
  uint8_t cmd[9] = {0xFF, 0x01, 0x86, 0x00, 0x00, 0x00, 0x00, 0x00, 0x79}; 
  uint8_t response[9];
  uint8_t cnt = 0;
  
  while (true) {
    //uint16_t c = uart_getc();
    //uint8_t cmd = uartWaitNextChar();

    for (uint8_t i = 0; i < sizeof(cmd); i++) {
      uart_putc(cmd[i]);
    }
    for (uint8_t i = 0; i < 9; i++) {
      response[i] = uartWaitNextChar();
    }
  
    uint8_t crc = 0;
    for (uint8_t i = 1; i < 8; i++) {
      crc += response[i];
    }
    crc = 255 - crc;
    crc++;

    if (response[8] != crc || response[1] != 0x86 || response[0] != 0xff ) {
//      wdt_enable(WDTO_4S);
    } else {
//      wdt_reset();
    }
    

    selectFont(FONT_5x7);
    drawValue(0, "resp[0]", response[0], response[0] == 0xff ? COLOR_GREEN : COLOR_RED);
    drawValue(10, "resp[1]", response[1], response[1] == 0x86 ? COLOR_GREEN : COLOR_RED);
    drawValue(20, "crc-read", response[8], response[8] == crc ? COLOR_GREEN : COLOR_RED);
    drawValue(30, "crc-calc", crc, response[8] == crc ? COLOR_GREEN : COLOR_RED);
    drawValue(110, "N", cnt++, COLOR_BLUE);

    uint16_t ppm = (response[2] << 8) + response[3];
    drawValue(70, " ppm", ppm, COLOR_MAGENTA);


    _delay_ms(1000);
//  if ( !(response[0] == 0xFF && response[1] == 0x86 && response[8] == crc) ) {
//    Serial.println("CRC error: " + String(crc) + " / "+ String(response[8]));
//  } else {
//    unsigned int responseHigh = (unsigned int) response[2];
//    unsigned int responseLow = (unsigned int) response[3];
//    unsigned int ppm = (256*responseHigh) + responseLow;
//    Serial.println(ppm);
//  }
//  delay(10000);

  }


 */

AirQuality airqualitysensor;
MHZ19 mhz(&Serial);
DHT dht(DHTPIN, DHTTYPE);

int current_quality =-1;


rgb_lcd lcd;

void setup() 
{
    Serial.begin(9600);
    // set up the LCD's number of columns and rows:
    lcd.begin(16, 2);
    lcd.print("Initialization");
    lcd.setCursor(0, 1);
    lcd.print("Please wait...");

    airqualitysensor.init(14);
    
//    lcd.setRGB(colorR, colorG, colorB);
    
    // Print a message to the LCD.
    dht.begin();
    delay(1000);
    mhz.setRange(MHZ19_RANGE_2000);
    mhz.setAutoCalibration(true);
}

void printCo2() {
  MHZ19_RESULT response = mhz.retrieveData();
  lcd.setCursor(8, 0);
  if (response == MHZ19_RESULT_OK)  {
    lcd.print(F("CO2 "));
    lcd.print(mhz.getCO2());
    lcd.print(F("  "));
//    lcd.print(F(" T:"));
//    lcd.print(mhz.getTemperature());
    //lcd.print(F(" A:"));
    //lcd.print(mhz.getAccuracy());
  }
  else
  {
    lcd.print(F("Error, code: "));
    lcd.print(response);
  }
  
}


void printDht() {
  // Reading temperature or humidity takes about 250 milliseconds!
  // Sensor readings may also be up to 2 seconds 'old' (its a very slow sensor)
  float h = dht.readHumidity();
  // Read temperature as Celsius (the default)
  float t = dht.readTemperature();

  lcd.setCursor(0, 1);
  // Check if any reads failed and exit early (to try again).
  if (isnan(h) || isnan(t)) {
    lcd.print("DHT Error!");
    return;
  }

  // Compute heat index in Celsius (isFahreheit = false)
  float hic = dht.computeHeatIndex(t, h, false);

  lcd.print(F("H               "));
  lcd.setCursor(2, 1);
  lcd.print(h, 1);
  lcd.print(F("%"));

  lcd.setCursor(11, 1);
  //lcd.print(F("T "));
  lcd.print(t, 1);
  lcd.print(F("C"));
//  Serial.print(F("°F  Heat index: "));
//  Serial.print(hic);
//  Serial.print(F("°C "));
//  Serial.print(hif);
//  Serial.println(F("°F"));
  
}


void printPol() {
    current_quality = airqualitysensor.slope();
    lcd.setCursor(0, 0);
    lcd.print("Pol     ");
    lcd.setCursor(4, 0);
    lcd.print(airqualitysensor.first_vol);
}

void loop() 
{
  printPol();
  printCo2();
  printDht();

//    if (current_quality >= 0) {// if a valid data returned.
//      lcd.setCursor(0, 0);
//      lcd.setCursor(0, 1);
//        if (current_quality==0)
//            lcd.print("Crit. pollution!");
//        else if (current_quality==1)
//            lcd.print("High pollution! ");
//        else if (current_quality==2)
//            lcd.print("Low pollution!  ");
//        else if (current_quality ==3)
//            lcd.print("Fresh air       ");
//    }


  
    // print the number of seconds since reset:
//    lcd.print(millis()/1000);

  
    delay(100);
}

ISR(TIMER1_OVF_vect)
{
  if(airqualitysensor.counter==61)//set 2 seconds as a detected duty
  {

      airqualitysensor.last_vol=airqualitysensor.first_vol;
      airqualitysensor.first_vol = analogRead(A0);
      airqualitysensor.counter=0;
      airqualitysensor.timer_index=1;
      PORTB = PORTB ^ 0x20;
  }
  else
  {
    airqualitysensor.counter++;
  }
}
